/*
    Scanbot SDK Flutter Plugin

    Copyright (c) 2019 doo GmbH. All rights reserved.
 */
package io.scanbot.sdk.flutter

import android.app.Activity
import android.content.Intent
import io.flutter.plugin.common.PluginRegistry
import kotlinx.coroutines.CompletableDeferred
import kotlinx.coroutines.Deferred

public interface ICoroutineActivityResultHandler {
    var currentCode: Int
    var resultByCode: MutableMap<Int, CompletableDeferred<ActivityResult?>>

    /**
     * Launches the intent allowing to process the result using await()
     *
     * @param intent the intent to be launched.
     *
     * @return Deferred<ActivityResult>
     */
    fun launchIntent(
        activity: Activity,
        intent: Intent
    ): Deferred<ActivityResult?> {
        val activityResult = CompletableDeferred<ActivityResult?>()

        if (intent.resolveActivity(activity.packageManager) != null) {
            val resultCode = currentCode++
            resultByCode[resultCode] = activityResult
            activity.startActivityForResult(intent, resultCode)
        } else {
            activityResult.complete(null)
        }
        return activityResult
    }
}

open class CoroutineResultHandler(
) : ICoroutineActivityResultHandler, PluginRegistry.ActivityResultListener {
    override var currentCode: Int = 0
    override var resultByCode = mutableMapOf<Int, CompletableDeferred<ActivityResult?>>();

    override fun onActivityResult(requestCode: Int, resultCode: Int, data: Intent?): Boolean {
        resultByCode[requestCode]?.let {
            it.complete(ActivityResult(resultCode, data))
            resultByCode.remove(requestCode)
        }
        return false
    }

}


/**
 * Wraps the parameters of onActivityResult
 *
 * @property resultCode the result code returned from the activity.
 * @property data the optional intent returned from the activity.
 */
class ActivityResult(
    val resultCode: Int,
    val data: Intent?
)
