/*
    Scanbot SDK Flutter Plugin

    Copyright (c) 2019 doo GmbH. All rights reserved.
 */
package io.scanbot.sdk.flutter

import io.flutter.plugin.common.MethodCall
import io.flutter.plugin.common.MethodChannel

interface ISdkSubplugin {
    suspend fun onMethodCall(call: MethodCall, result: MethodChannel.Result): Boolean
}

interface ISdkSubpluginManager {

    val subplugins: MutableSet<ISdkSubplugin>
    fun attachSubplugin(subplugin: ISdkSubplugin) {
        subplugins.removeAll { it::class.java == subplugin::class.java }
        subplugins.add(subplugin)
    }

    fun <T> detachSubpluginByType(type: Class<T>) {
        subplugins.removeAll { it::class.java == type }
    }

    suspend fun callMethod(call: MethodCall, result: MethodChannel.Result): Boolean {
        return subplugins.firstOrNull { it.onMethodCall(call, result) }.takeIf { it != null }?.let { true }
                ?: false
    }

    fun clearPlugins() = subplugins.clear()
}

class SdkSubpluginManager : ISdkSubpluginManager {
    override val subplugins: MutableSet<ISdkSubplugin> = mutableSetOf()
}