/*
    Scanbot SDK Flutter Plugin

    Copyright (c) 2019 doo GmbH. All rights reserved.
 */
package io.scanbot.sdk.flutter

import android.net.Uri
import android.util.Base64
import androidx.core.net.toUri
import com.google.gson.Gson
import com.google.gson.JsonObject
import com.google.gson.TypeAdapter
import com.google.gson.stream.JsonReader
import com.google.gson.stream.JsonWriter
import io.scanbot.sdk.flutter.camera.config.BarcodeScannerConfiguration
import io.scanbot.sdk.flutter.camera.config.FinderConfig
import io.scanbot.sdk.flutter.utils.JsonUtils
import io.scanbot.sdk.ocr.process.OcrResult
import io.scanbot.sdk.persistence.Page
import org.json.JSONObject
import java.io.File
import java.text.SimpleDateFormat
import java.util.*

fun ocrResultToJson(ocrResult: OcrResult, newFile: File?): JSONObject {
    val ocrResultJson = JsonUtils.resultToJson(ocrResult)

    ocrResult.sandwichedPdfDocumentFile?.toUri()?.toString()?.let {
        ocrResultJson.put(
                "pdfFileUri",
                it
        )
    }

    newFile?.toUri()?.toString()?.let {
        ocrResultJson.put(
                "pdfFileUri",
                it
        )
    }

    ocrResult.recognizedText?.let {
        ocrResultJson.put("plainText", it)
    }

    return ocrResultJson
}

typealias PagesJson = List<HashMap<String, Any>>
typealias Json = HashMap<String, Any>
typealias LanguagesJson = ArrayList<String>

fun Json.toPage(gson: Gson): Page {
    return gson.fromJson(JSONObject(this as Map<*, *>).toString(), Page::class.java)
}

fun Json.toFinderConfig(gson: Gson): FinderConfig {
    return gson.fromJson(JSONObject(this as Map<*, *>).toString(), FinderConfig::class.java)
}

fun Json.toBarcodeScannerConfig(gson: Gson): BarcodeScannerConfiguration {
    return gson.fromJson(JSONObject(this as Map<*, *>).toString(), BarcodeScannerConfiguration::class.java)
}

fun Page.toJson(gson: Gson): JsonObject {
    val size = documentImageSizeLimit
    val jsonObject = gson.toJsonTree(this) as JsonObject
    if (size.width == Integer.MAX_VALUE && size.height == Integer.MAX_VALUE) {
        jsonObject.remove("documentImageSizeLimit")
    }
    (jsonObject.get("documentImageSizeLimit") as JsonObject?)?.let {
        it.addProperty("width", size.width)
        it.addProperty("height", size.height)
    }

    return jsonObject
}

fun Json.toSize(gson: Gson): Page.Size {
    return gson.fromJson(JSONObject(this as Map<*, *>).toString(), Page.Size::class.java)
}

// Using Android's base64 libraries. This can be replaced with any base64 library.
class ByteArrayToBase64TypeAdapter : TypeAdapter<ByteArray>() {
    override fun write(out: JsonWriter?, value: ByteArray?) {
        out?.value(value?.let { Base64.encodeToString(it, Base64.NO_WRAP) } ?: "")
    }

    override fun read(inner: JsonReader?): ByteArray {
        return ByteArray(0)// we do not have any byte arrays going in sdk
    }
}


fun fixNfcResultForFlutter(resultJson: JsonObject, photoUri: Uri?) {
    val inFormat = SimpleDateFormat("yyMMdd")
    val outFormat = SimpleDateFormat("yyyy-MM-dd")

    (resultJson["dg1Group"] as? JsonObject)?.apply {
        get("checkDigitDocNumber")?.asInt?.let { addProperty("documentNumberCheckDigit", it) }
        get("dateOfBirth6Digit")?.asString?.let { addProperty("dateOfBirth", outFormat.format(inFormat.parse(it))) }
        get("dateOfExpiry")?.asString?.let { addProperty("dateOfExpiry", outFormat.format(inFormat.parse(it))) }
        get("checkDigitDateOfBirth")?.asInt?.let { addProperty("dateOfBirthCheckDigit", it) }
        get("checkDigitExpiryDate")?.asInt?.let { addProperty("dateOfExpiryCheckDigit", it) }
        get("checkDigitComposite")?.asInt?.let { addProperty("compositeCheckDigit", it) }
        get("checkDigitOptionalData")?.asInt?.let { addProperty("optionalDataCheckDigitTD3", it) }

    }
    (resultJson["dg2Group"] as? JsonObject)?.apply {
        get("facialRecordDataLength")?.asInt?.let { addProperty("lengthOfFacialRecord", it) }
        get("imageColorspace")?.asInt?.let { addProperty("imageColorSpace", it) }
    }
    resultJson.add("dataGroup1", resultJson["dg1Group"])
    resultJson.add("dataGroup2", resultJson["dg2Group"])
    photoUri?.toString()?.let { resultJson.addProperty("photoUri", it) }
    resultJson.remove("dg1Group")
    resultJson.remove("dg2Group")
    resultJson.remove("photo")
}
