package io.scanbot.sdk.flutter.camera.config

import io.scanbot.sdk.barcode.entity.*
import io.scanbot.sdk.ui.view.barcode.configuration.BarcodeImageGenerationType

class BarcodeScannerConfiguration {

    /// List of accepted Barcode Formats; any formats that are not included in this
    /// list will not be detected by the `BarcodeCameraView`. By default, the list
    /// includes all supported barcode types.
    var barcodeFormats: List<BarcodeFormat>? = null

    /// List of accepted Document Formats; any document formats that are not included
    /// in this list will be ignored by the `BarcodeCameraView`. By default, the list
    /// includes all supported document types.
    var acceptedDocumentFormats: List<BarcodeDocumentFormat>? = null

    /// The engine mode of the barcode recognizer. Defaults to NEXT_GEN.
    /// To use legacy recognizer, please specify LEGACY
    var engineMode: EngineMode? = null

    /// Additional parameters for tweaking the detection of barcodes.
    var additionalParameters: BarcodeScannerAdditionalConfiguration? = null


    var detectionIntervalInMs: Int? = null

    /// Parameter that declare whether we need to make snapshot of whole page when we successfully snapped any barcode
    var barcodeImageGenerationType: BarcodeImageGenerationType? = null

    /// Changes sensitivity of auto-snapping. That is: the more sensitive it is the faster it shoots.
    /// This parameter only works when `barcodeImageGenerationType = BarcodeImageGenerationType.CAPTURED_IMAGE`
    /// Sensitivity must be within [0..1] range. A value of 1.0 triggers automatic capturing immediately,
    /// a value of 0.0 delays the automatic capturing by 3 seconds.
    ///
    /// The default value is 0.66 (1 sec) so you will get snapped barcode result with saved image each not offen than 1 second.
    ///
    /// @param sensitivity should be within [0..1] range.
    var autoSnappingSensitivity: Float? = null
}

class BarcodeScannerAdditionalConfiguration {
    var enableGS1Decoding: Boolean? = null

    var stripCheckDigits: Boolean? = null

    var maximumTextLength: Int? = null

    var minimumTextLength: Int? = null

    var minimum1DBarcodesQuietZone: Int? = null

    var msiPlesseyChecksumAlgorithm : MSIPlesseyChecksumAlgorithm? = null
}


enum class DetectorType {
    barcode, mrz, ehic, document, payform
}
