package io.scanbot.sdk.flutter.camera.usecase

import android.graphics.Bitmap
import io.scanbot.sdk.persistence.BarcodeFileStorage
import io.scanbot.sdk.persistence.fileio.ImageFileIOProcessor
import io.scanbot.sdk.util.singleFlow
import kotlinx.coroutines.flow.Flow
import java.io.File
import java.util.*

/**
 * Saves preview frames in the files repository
 */
class SaveBarcodePreviewFrameUseCase(private val barcodeFileStorage: BarcodeFileStorage,
                                     private val imageFileIOProcessor: ImageFileIOProcessor) {

    /**
     * Stores the preview frame in [BarcodeFileStorage]
     * throws [IOException] that need to be handled with catch {} in flow api
     */
    fun savePreviewFrame(
            previewFrame: Bitmap
    ): Flow<String> {
        return singleFlow {
            val file = File(barcodeFileStorage.getBarcodeImagesDirectory().path + "/" + UUID.randomUUID().toString() + ".jpg")

            imageFileIOProcessor.writeImage(previewFrame, Bitmap.CompressFormat.JPEG, 100, file)

            file.path
        }
    }
}
