package io.scanbot.sdk.flutter.camera.usecase

import android.graphics.Bitmap
import android.graphics.BitmapFactory
import android.graphics.Matrix
import io.scanbot.sdk.persistence.BarcodeFileStorage
import io.scanbot.sdk.persistence.fileio.ImageFileIOProcessor
import io.scanbot.sdk.util.singleFlow
import kotlinx.coroutines.flow.Flow
import java.io.File
import java.util.*

/**
 * Saves taken camera pictures in the files repository
 */
class SaveBarcodeTakenPictureUseCase(private val barcodeFileStorage: BarcodeFileStorage,
                                     private val imageFileIOProcessor: ImageFileIOProcessor) {

    /**
     * Stores taken picture in [BarcodeFileStorage]
     * throws [IOException] and [Exception] that need to be handled with catch {} in flow api
     */
    fun saveTakenPicture(
            image: ByteArray,
            imageOrientation: Int = 0
    ): Flow<String> {
        return singleFlow {
            val file = File(barcodeFileStorage.getBarcodeImagesDirectory().path + "/" + UUID.randomUUID().toString() + ".jpg")

            val options = BitmapFactory.Options()
            var resultBitmap = BitmapFactory.decodeByteArray(image, 0, image.size, options)
            if (imageOrientation != 0) {
                val matrix = Matrix()
                matrix.postRotate(imageOrientation.toFloat())
                resultBitmap =
                        Bitmap.createBitmap(resultBitmap, 0, 0, resultBitmap.width, resultBitmap.height, matrix, false)
            }

            imageFileIOProcessor.writeImage(resultBitmap, Bitmap.CompressFormat.JPEG, 100, file)

            file.path

        }
    }
}
