package io.scanbot.sdk.flutter.utils

import android.content.Context


/**
 * This method converts dp unit to equivalent pixels, depending on device density.
 *
 * @param dp      A value in dp (density independent pixels) unit. Which we need to convert into pixels
 * @param context Context to get resources and device specific display metrics
 * @return A float value to represent px equivalent to dp depending on device density
 */
public fun convertDpToPx(context: Context, dp: Float): Float {
    return dp * context.resources.displayMetrics.density;
}

/**
 * This method converts device specific pixels to density independent pixels.
 *
 * @param px      A value in px (pixels) unit. Which we need to convert into db
 * @param context Context to get resources and device specific display metrics
 * @return A float value to represent dp equivalent to px value
 */
public fun convertPxToDp(context: Context, px: Float): Float {
    return px / context.resources.displayMetrics.density;
}

