//
//  OCRPerformer.swift
//  scanbot_sdk
//
//  Created by Zhenya Knizhnik on 02.09.19.
//

import Foundation
import ScanbotSDK
import Flutter

class OCRPerformer {
    func perform(with config: OCRPerformConfiguration, result: @escaping FlutterResult) {
        
        let storage = SBSDKIndexedImageStorage.temporaryStorage(for: config.pages)
        var outputPDFURL: URL?
        if config.shouldGeneratePdf {
            outputPDFURL = PluginPreferences.shared.newStorageCustomFileURL(extensionString: "pdf")
        }

        SBSDKOpticalTextRecognizer.recognizeText(storage,
                                                 copyImageStorage: false,
                                                 indexSet: nil,
                                                 languageString: config.languages,
                                                 encrypter: ScanbotSDKUI.defaultPDFEncrypter(),
                                                 pdfOutputURL: outputPDFURL) { (finished, error, info) in
            if let error = error {
                result(FlutterError(code: "1012", message: "ERROR_iOS: " + error.localizedDescription, details: "OCR error"))
            } else if finished,
                let ocrResult = info?[SBSDKResultInfoOCRResultsKey] as? SBSDKOCRResult {
                    let outputURL = info?[SBSDKResultInfoDestinationFileURLKey] as? URL
                    let ocrResult = OCRResult(sdkORCResult: ocrResult, pdfURL: outputURL)
                    result(ocrResult.encodedString)
            } else {
                result(FlutterError(code: "1013", message: "ERROR_iOS: Unknown rendering error", details: "OCR error"))
            }
            storage.removeAllImages()
        }
    }
}


