//
//  BarcodeBatchDataMapper.swift
//  scanbot_sdk
//
//  Created by Yevgeniy Knizhnik on 03.02.21.
//

import Foundation
import ScanbotSDK
import Flutter

class BarcodeBatchDataMapper: SBSDKUIBarcodesBatchScannerMapping {
    
    private weak var dataChannel: FlutterMethodChannel?
    
    init(dataChannel: FlutterMethodChannel?) {
        self.dataChannel = dataChannel
    }
    
    func loadMappedData(forBarcode barcode: SBSDKBarcodeScannerResult,
                        completion: ((SBSDKUIBarcodeMappedData?) -> Void)? = nil) {
        self.dataChannel?.invokeMethod("getFormattedData",
                                       arguments: barcode.encodedDictionary.encodedString,
                                       result: { response in
                                        
            DispatchQueue(label: "scanbot.barcode.flutter.sdk.mapping").async {
                guard let arguments = response as? [String: Any],
                let mappedObject = SBSDKUIBarcodeMappedData(from: arguments) else {
                    let defaultMappedObject = SBSDKUIBarcodeMappedData(name: barcode.rawTextString,
                                                                       description: barcode.type.name)

                    completion?(defaultMappedObject)
                    return;
                }
                completion?(mappedObject)
            }
        })
    }
}
