//
//  SBSDKUILicensePlateScannerUIConfiguration.swift
//  scanbot_sdk
//
//  Created by Yevgeniy Knizhnik on 16.04.21.
//

import Foundation
import ScanbotSDK

extension SBSDKUILicensePlateScannerUIConfiguration {
    convenience init(from dictionary: [String: Any]) {
        self.init()
        if let hex = dictionary["topBarButtonsColor"] as? String {
            self.topBarButtonsColor = UIColor(hexString: hex)
        }
        
        if let hex = dictionary["topBarBackgroundColor"] as? String {
            self.topBarBackgroundColor = UIColor(hexString: hex)
        }
        
        if let hex = dictionary["topBarButtonsInactiveColor"] as? String {
            self.topBarButtonsInactiveColor = UIColor(hexString: hex)
        }
        
        if let hex = dictionary["cameraOverlayColor"] as? String {
            self.cameraOverlayColor = UIColor(hexString: hex)
        }
        
        if let hex = dictionary["finderLineColor"] as? String {
            self.finderLineColor = UIColor(hexString: hex)
        }
        
        if let hex = dictionary["finderTextHintColor"] as? String {
            self.finderTextHintColor = UIColor(hexString: hex)
        }
        
        if let finderLineWidth = dictionary["finderLineWidth"] as? CGFloat {
            self.finderLineWidth = finderLineWidth
        }
        
        if let string = dictionary["orientationLockMode"] as? String {
            let allowedInterfaceOrientations = UIInterfaceOrientationMask(from: string)
            self.allowedInterfaceOrientations = allowedInterfaceOrientations
        }
        
        /**
         Mapping android logic here.
         Parameters:
         1. confirmationDialogAccentColor - color for the text of retry and confirm buttons.
         2. confirmationDialogConfirmButtonFilled - for confirm button color of the text becomes background color and vice versa. Works after applying accent.
         3. confirmationDialogConfirmButtonFilledTextColor - ignored, if confirmationDialogConfirmButtonFilled != true. Confirm button text color otherwise.
         */
        
        if let hex = dictionary["confirmationDialogAccentColor"] as? String {
            let accentColor = UIColor(hexString: hex)
            self.confirmationDialogConfirmButtonStyle.textColor = accentColor
            self.confirmationDialogRetryButtonStyle.textColor = accentColor
        }
        
        var didInvertColors = false
        if let isConfirmationButtonFilled = dictionary["confirmationDialogConfirmButtonFilled"] as? Bool,
           isConfirmationButtonFilled == true {
            didInvertColors = true
            let backgroundColor = self.confirmationDialogConfirmButtonStyle.textColor
            let textColor = self.confirmationDialogConfirmButtonStyle.backgroundColor
            self.confirmationDialogConfirmButtonStyle.textColor = textColor
            self.confirmationDialogConfirmButtonStyle.backgroundColor = backgroundColor
            self.confirmationDialogConfirmButtonStyle.highlightedBackgroundColor
                = self.confirmationDialogConfirmButtonStyle.backgroundColor.withAlphaComponent(0.75)
        }
        
        if let hex = dictionary["confirmationDialogConfirmButtonFilledTextColor"] as? String,
           didInvertColors == true {
            self.confirmationDialogConfirmButtonStyle.textColor = UIColor(hexString: hex)
        }
        
        self.confirmationDialogConfirmButtonStyle.highlightedTextColor = self.confirmationDialogConfirmButtonStyle.textColor
        self.confirmationDialogRetryButtonStyle.highlightedTextColor = self.confirmationDialogRetryButtonStyle.textColor
    }
}
