//
//  SBSDKBarcodeType.swift
//  scanbot_sdk
//
//  Created by Yevgeniy Knizhnik on 04.05.20.
//

import Foundation
import ScanbotSDK

extension SBSDKBarcodeType {
    static func fromArguments(_ arguments: [String: Any]) -> [SBSDKBarcodeType]? {
        if let stringArray = arguments["barcodeFormats"] as? [String] {
            let barcodes = stringArray.compactMap({ SBSDKBarcodeType.from(string: $0) })
            return barcodes
        }
        return nil
    }
    
    static func from(string: String) -> SBSDKBarcodeType? {
        switch string {
        case "AZTEC": return SBSDKBarcodeTypeAztec
        case "CODABAR": return SBSDKBarcodeTypeCodaBar
        case "CODE_39": return SBSDKBarcodeTypeCode39
        case "CODE_93": return SBSDKBarcodeTypeCode93
        case "CODE_128": return SBSDKBarcodeTypeCode128
        case "DATA_MATRIX": return SBSDKBarcodeTypeDataMatrix
        case "EAN_8": return SBSDKBarcodeTypeEAN8
        case "EAN_13": return SBSDKBarcodeTypeEAN13
        case "ITF": return SBSDKBarcodeTypeITF
        case "PDF_417": return SBSDKBarcodeTypePDF417
        case "QR_CODE": return SBSDKBarcodeTypeQRCode
        case "RSS_14": return SBSDKBarcodeTypeRSS14
        case "RSS_EXPANDED": return SBSDKBarcodeTypeRSSExpanded
        case "UPC_A": return SBSDKBarcodeTypeUPCA
        case "UPC_E": return SBSDKBarcodeTypeUPCE
        case "MSI_PLESSEY": return SBSDKBarcodeTypeMSIPlessey
        case "UNKNOWN": return SBSDKBarcodeTypeUnknown
        default: return nil
        }
    }
    
    var encodedString: String? {
        switch self {
        case SBSDKBarcodeTypeAztec: return "AZTEC"
        case SBSDKBarcodeTypeCodaBar: return "CODABAR"
        case SBSDKBarcodeTypeCode39: return "CODE_39"
        case SBSDKBarcodeTypeCode93: return "CODE_93"
        case SBSDKBarcodeTypeCode128: return "CODE_128"
        case SBSDKBarcodeTypeDataMatrix: return "DATA_MATRIX"
        case SBSDKBarcodeTypeEAN8: return "EAN_8"
        case SBSDKBarcodeTypeEAN13: return "EAN_13"
        case SBSDKBarcodeTypeITF: return "ITF"
        case SBSDKBarcodeTypePDF417: return "PDF_417"
        case SBSDKBarcodeTypeQRCode: return "QR_CODE"
        case SBSDKBarcodeTypeRSS14: return "RSS_14"
        case SBSDKBarcodeTypeRSSExpanded: return "RSS_EXPANDED"
        case SBSDKBarcodeTypeUPCA: return "UPC_A"
        case SBSDKBarcodeTypeUPCE: return "UPC_E"
        case SBSDKBarcodeTypeMSIPlessey: return "MSI_PLESSEY"
        case SBSDKBarcodeTypeUnknown: return "UNKNOWN"
        default: return nil
        }
    }
}
