//
//  SBSDKUIPage.swift
//  scanbot_sdk
//
//  Created by Zhenya Knizhnik on 22.08.19.
//

import Foundation
import ScanbotSDK

extension SBSDKUIPage {
    var encodedDictionary: [String: Any?] {
        var dictionary: [String: Any?] = ["filter": self.filter.stringName,
                                          "pageId": self.pageFileUUID.uuidString,
                                          "polygon": self.polygon?.encodedDictionary ?? [],
                                          "originalImageFileUri": self.originalImageURL()?.absoluteString ?? "",
                                          "documentImageFileUri": self.documentImageURL()?.absoluteString ?? "",
                                          "originalPreviewImageFileUri": self.documentPreviewImageURL()?.absoluteString ?? "",
                                          "documentPreviewImageFileUri": self.documentPreviewImageURL()?.absoluteString ?? ""]
        if let status = self.status.stringName {
            dictionary["detectionStatus"] = status
        }
        if self.documentImageSizeLimit != .zero {
            dictionary["documentImageSizeLimit"] = self.documentImageSizeLimit.encodedDictionary
        }
        return dictionary
    }
    
    var encodedString: String? {
        return self.encodedDictionary.encodedString
    }
    
    convenience init?(from dictionary: [String: Any]) {
        let pageDictionary = dictionary["page"] as? [String: Any?] ?? dictionary
        guard let id = pageDictionary["pageId"] as? String,
            let uuid = UUID(uuidString: id) else {
                return nil
        }
        
        let polygon = SBSDKPolygon(from: pageDictionary)
        if let detectionStatus = pageDictionary["detectionStatus"] as? String {
            polygon?.detectionStatus = SBSDKDocumentDetectionStatus(statusString: detectionStatus)
        }
        
        var filter = SBSDKImageFilterTypeNone
        if let filterString = pageDictionary["filter"] as? String {
            filter = SBSDKImageFilterType(string: filterString)
        }
        
        var size = CGSize.zero
        if let sizeDictionary = pageDictionary["documentImageSizeLimit"] as? [String: Any] {
            size = CGSize(from: sizeDictionary)
        }
        
        self.init(pageFileID: uuid, polygon: polygon, filter: filter, documentImageSizeLimit: size)
    }
    
    static func pages(from dictionary: [String: Any]) -> [SBSDKUIPage] {
        guard let pagesArray = dictionary["pages"] as? [[String: Any]] else {
            return []
        }
        
        var pages: [SBSDKUIPage] = []
        pagesArray.forEach { (pageDictionary) in
            if let page = SBSDKUIPage(from: pageDictionary) {
                pages.append(page)
            }
        }
        return pages
    }
}
