//
//  CallErrorHandler.swift
//  scanbot_sdk
//
//  Created by Yevgeniy Knizhnik on 15.06.21.
//

import Foundation

protocol CallErrorHandler {
    func returnArgumentsError(call: FlutterMethodCall, result: @escaping FlutterResult)
    func returnCustomError(call: FlutterMethodCall, message: String, result: @escaping FlutterResult)
    func returnUnknownCallError(call: FlutterMethodCall, result: @escaping FlutterResult)
}

extension CallErrorHandler {
    func returnArgumentsError(call: FlutterMethodCall, result: @escaping FlutterResult) {
        DispatchQueue.main.async {
            result(FlutterError(code: "1001",
                                message: "ERROR_iOS: \(call.method) was called with wrong arguments",
                                details: call.arguments))
        }
    }
    
    func returnUnknownCallError(call: FlutterMethodCall, result: @escaping FlutterResult) {
        DispatchQueue.main.async {
            result(FlutterError(code: "1002",
                                message: "ERROR_iOS: \(call.method) is unknown or can't be handled",
                                details: nil))
        }
    }
    
    func returnCustomError(call: FlutterMethodCall, message: String, result: @escaping FlutterResult) {
        DispatchQueue.main.async {
            result(FlutterError(code: "1003",
                                message: "ERROR_iOS: \(message)",
                                details: call.arguments))
        }
    }
}
