//
//  BatchScannerPresenter.swift
//  scanbot_sdk
//
//  Created by Yevgeniy Knizhnik on 03.02.21.
//

import Foundation
import ScanbotSDK
import Flutter

class BatchScannerPresenter: NSObject {
    private var resultHandler: FlutterResult?
    private weak var presenter: UIViewController?
    private var dataMapper: SBSDKUIBarcodesBatchScannerMapping?

    func present(on presenter: UIViewController,
                 with configuration: SBSDKUIBarcodesBatchScannerConfiguration,
                 dataMapper: SBSDKUIBarcodesBatchScannerMapping?,
                 barcodeTypes: [SBSDKBarcodeType]?,
                 barcodeDocumentTypes: [SBSDKBarcodeDocumentType]?,
                 result: @escaping FlutterResult) {
        self.resultHandler = result
        self.presenter = presenter
        
        var controller: SBSDKUIBarcodesBatchScannerViewController?
        if let documentTypes = barcodeDocumentTypes, !documentTypes.isEmpty {
            controller = SBSDKUIBarcodesBatchScannerViewController.createNew(withAcceptedMachineCodeTypes: barcodeTypes,
                                                                       withAcceptedDocumentTypes: barcodeDocumentTypes,
                                                                       configuration: configuration,
                                                                       andDelegate: self)
        } else {
            controller = SBSDKUIBarcodesBatchScannerViewController.createNew(withAcceptedMachineCodeTypes: barcodeTypes,
                                                                       configuration: configuration,
                                                                       andDelegate: self)
        }
        
        guard let controller = controller else {
            return
        }

        self.dataMapper = dataMapper
        controller.modalPresentationStyle = .fullScreen
        presenter.present(controller, animated: true) { [weak controller, weak dataMapper] in
            controller?.dataMapper = dataMapper
        }
    }
    
    private func processResult(barcodeResults: [SBSDKBarcodeScannerResult]) {
        let barcodeList = barcodeResults.map({$0.encodedDictionary})
        let result: [String: Any?] = ["barcodeItems": barcodeList]
        self.resultHandler?(result.encodedString)
        self.clean()
    }
    
    private func clean() {
        self.resultHandler = nil
        self.presenter = nil
        self.dataMapper = nil
    }
}

extension BatchScannerPresenter: SBSDKUIBarcodesBatchScannerViewControllerDelegate {
    func barcodesBatchScannerViewController(_ viewController: SBSDKUIBarcodesBatchScannerViewController,
                                            didFinishWith barcodeResults: [SBSDKUIBarcodeMappedResult]) {
        viewController.dataMapper?.loadMappedData(forBarcode: barcodeResults.first!.barcode, completion: nil)
        self.presenter?.dismiss(animated: true, completion: {
            self.processResult(barcodeResults: barcodeResults.map({ $0.barcode }))
        })
    }
    
    func barcodesBatchScannerViewControllerDidCancel(_ viewController: SBSDKUIBarcodesBatchScannerViewController) {
        self.clean()
    }
}
