//
//  SDKInitializationConfig.swift
//  Runner
//
//  Created by Zhenya Knizhnik on 22.08.19.
//  Copyright © 2019 The Chromium Authors. All rights reserved.
//

import Foundation
import ScanbotSDK

struct SDKInitializationConfig {
    enum ImageFormat {
        case png
        case jpeg(quality: Int)
        
        init(from dictionary: [String: Any]) {
            var format = ImageFormat.jpeg(quality: 80)
            if let formatString = dictionary["imageFormat"] as? String {
                switch formatString {
                case "PNG":
                    format = .png
                case "JPG":
                    let quality = dictionary["imageQuality"] as? Int ?? 80
                    format = .jpeg(quality: quality)
                default: break
                }
            }
            self = format
        }
    }
    
    let isLoggingEnabled: Bool
    let licenseCode: String
    let imageFormat: ImageFormat
    let storageBaseDirectory: URL?
    let detectorMode: SBSDKDocumentDetectorMode?
    let defaultEncrypter: SBSDKAESEncrypter?
    
    init?(from dictionary: [String: Any]) {
        guard let licenseCode = dictionary["licenseKey"] as? String else {
            return nil
        }
        self.licenseCode = licenseCode
        self.isLoggingEnabled = (dictionary["loggingEnabled"] as? Bool) ?? false
        self.imageFormat = ImageFormat(from: dictionary)
        
        if let detectorModeString = dictionary["documentDetectorMode"] as? String {
            self.detectorMode = SBSDKDocumentDetectorMode(from: detectorModeString)
        } else {
            self.detectorMode = nil
        }
        
        if let url = dictionary["storageBaseDirectory"] as? String {
            self.storageBaseDirectory = URL(fileURLWithPath: url, isDirectory: true)
        } else {
            self.storageBaseDirectory = nil
        }
        
        if let encryptionParameters = dictionary["encryptionParameters"] as? [String: Any] {
            self.defaultEncrypter = SBSDKAESEncrypter(from: encryptionParameters)
        } else {
            self.defaultEncrypter = nil
        }
    }
}
