//
//  ScanbotSDKInitializer.swift
//  Runner
//
//  Created by Zhenya Knizhnik on 22.08.19.
//  Copyright © 2019 The Chromium Authors. All rights reserved.
//

import Foundation
import ScanbotSDK

enum ScanbotSDKInitializer {
    private static var pluginDirectoryName = "sbsdk-plugin-storage"
    
    static func initialize(config: SDKInitializationConfig, result: @escaping FlutterResult) {
        if (!config.licenseCode.isEmpty) {
            ScanbotSDK.setLicense(config.licenseCode)
        }
        ScanbotSDK.setLoggingEnabled(config.isLoggingEnabled)
        
        if let detectorMode = config.detectorMode {
            SBSDKDocumentDetector.setDefaultDetectorMode(detectorMode)
        }
        
        if let encrypter = config.defaultEncrypter {
            ScanbotSDKUI.setDefaultImageStoreEncrypter(encrypter)
            ScanbotSDKUI.setDefaultPDFEncrypter(encrypter)
        }
        
        self.setStorage(format: config.imageFormat, storageURL: config.storageBaseDirectory)
        
        result(nil)
    }
    
    static private func setStorage(format: SDKInitializationConfig.ImageFormat, storageURL: URL?) {
        let location = SBSDKStorageLocation(baseURL: storageURL)
        PluginPreferences.shared.storageCustomFilesURL = location?.baseURL?.appendingPathComponent(self.pluginDirectoryName,
                                                                                              isDirectory: true)
        switch format {
        case .jpeg(let quality):
            SBSDKUIPageFileStorage.setDefault(SBSDKUIPageFileStorage(jpegFileFormatAndCompressionQuality: quality,
                                                                     location: location))
        case .png:
            SBSDKUIPageFileStorage.setDefault(SBSDKUIPageFileStorage(imageFileFormat: SBSDKImageFileFormat.PNG,
                                                                     location: location))
        }
    }
}
