#!/bin/bash
set -ex

cd jenkins/scanbot-sdk-flutter-tests

if [ "$(uname)" == "Darwin" ]; then
    ##### specify the DEVICE_NAME parameter in the Jenkins job #####
    #DEVICE_NAME="iPhone 11 Pro Max"

    # number of spaces indicate which column to use in the awk print command
    num_spaces=$(grep -o ' ' <<<"$DEVICE_NAME" | grep -c .)

    # column of the device id
    id_column=$((num_spaces + 2))

    # get the device id based on the device name
    DEVICE_ID_UNFORMATTED=$(xcrun simctl list devices | grep "$DEVICE_NAME" | head -n 1 | awk '{print $'"$id_column"'}')
    DEVICE_ID=$(echo "$DEVICE_ID_UNFORMATTED" | sed 's/[()]//g')

    # insert github auth token and assets url
    sed -i '' "s!<GITHUB_TOKEN>!$GITHUB_TOKEN!g" integration_test/resources/constants.dart
    sed -i '' "s!<ASSETS_URL>!$ASSETS_URL!g" integration_test/resources/constants.dart

    # download flutter
    git clone https://github.com/flutter/flutter.git -b stable
    PATH="$PATH:$(pwd)/flutter/bin:$(pwd)/.pub-cache/bin:$HOME/.pub-cache/bin"
    export PATH
    
    # install dependencies
    cd ios 
    rm -rf Podfile.lock
    export LANG=en_US.UTF-8
    flutter pub get
    flutter precache --ios
    pod install
    pod update
    cd ..

elif [ "$(uname)" == "Linux" ]; then
    DEVICE_ID="emulator-$EMU_PORT"
            
    # insert github auth token and assets url            
    sed -i'' "s!<GITHUB_TOKEN>!$GITHUB_TOKEN!g" integration_test/resources/constants.dart
    sed -i'' "s!<ASSETS_URL>!$ASSETS_URL!g" integration_test/resources/constants.dart
fi

# run tests
flutter pub global activate junitreport
flutter test integration_test --device-id "$DEVICE_ID" --machine | tojunit > testreport.xml
