import 'dart:core';

import 'package:json_annotation/json_annotation.dart';
import 'package:scanbot_sdk/barcode_parsers/barcode_scanner.dart';

part 'boarding_pass.g.dart';

@JsonSerializable(
    createFactory: true, includeIfNull: false, explicitToJson: true)
class BoardingPassDocument extends BarcodeFormattedResult {
  static const String DOCUMENT_FORMAT = 'BoardingPass';

  /// Number of legs in document
  int numberOfLegs;

  /// Passenger name
  String name;

  /// Value indicates that this ticket is an electronic ticket
  bool electronicTicket;

  /// Security data string
  String securityData;

  /// Legs information
  List<BoardingPassLeg> legs;

  BoardingPassDocument(this.numberOfLegs, this.name, this.electronicTicket,
      this.securityData, this.legs)
      : super(DOCUMENT_FORMAT);

  factory BoardingPassDocument.fromJson(Map<String, dynamic> json) =>
      _$BoardingPassDocumentFromJson(json);

  @override
  Map<String, dynamic> toJson() => _$BoardingPassDocumentToJson(this);
}

@JsonSerializable(
    createFactory: true, includeIfNull: false, explicitToJson: true)
class BoardingPassLeg {
  /// Fields vector
  List<BoardingPassLegField> fields;

  BoardingPassLeg(this.fields);

  factory BoardingPassLeg.fromJson(Map<String, dynamic> json) =>
      _$BoardingPassLegFromJson(json);

  Map<String, dynamic> toJson() => _$BoardingPassLegToJson(this);
}

@JsonSerializable(
    createFactory: true, includeIfNull: false, explicitToJson: true)
class BoardingPassLegField {
  /// Field type
  BoardingPassDocumentFieldType type;

  /// Field value
  String value;

  BoardingPassLegField(this.type, this.value);

  factory BoardingPassLegField.fromJson(Map<String, dynamic> json) =>
      _$BoardingPassLegFieldFromJson(json);

  Map<String, dynamic> toJson() => _$BoardingPassLegFieldToJson(this);
}

/// Boarding pass document field type
enum BoardingPassDocumentFieldType {
  OperatingCarrierPNRCode,
  DepartureAirportCode,
  DestinationAirportCode,
  OperatingCarrierDesignator,
  FlightNumber,
  DateOfFlightJulian,
  CompartmentCode,
  SeatNumber,
  CheckInSequenceNumber,
  PassengerStatus,
  VersionNumber,
  PassengerDescription,
  SourceOfCheckIn,
  SourceOfBoardingPassIssuance,
  DateOfBoardingPassIssuanceJulian,
  DocumentType,
  AirlineDesignatorOfBoardingPassIssuer,
  BaggageTagLicensePlateNumbers,
  FirstNonConsecutiveBaggageTagLicensePlateNumber,
  SecondNonConsecutiveBaggageTagLicensePlateNumber,
  AirlineNumericCode,
  DocumentFormSerialNumber,
  SelecteeIndicator,
  InternationalDocumentationVerification,
  MarketingCarrierDesignator,
  FrequentFlyerAirlineDesignator,
  FrequentFlyerNumber,
  IDADIndicator,
  FreeBaggageAllowance,
  FastTrack,
  ForIndividualAirlineUse
}
