import 'package:json_annotation/json_annotation.dart';
import 'package:scanbot_sdk/barcode_parsers/barcode_scanner.dart';

part 'id_card_pdf_417.g.dart';

/// ID card PDF417 parsed barcode document
@JsonSerializable(
    createFactory: true, includeIfNull: false, explicitToJson: true)
class IDCardPDF417Document extends BarcodeFormattedResult {
  static const String DOCUMENT_FORMAT = 'IDCardPDF417';

  /// Fileds in the parsed document
  List<IDCardPDF417DocumentField> fields;

  IDCardPDF417Document(this.fields) : super(DOCUMENT_FORMAT);

  factory IDCardPDF417Document.fromJson(Map<String, dynamic> json) =>
      _$IDCardPDF417DocumentFromJson(json);
  @override
  Map<String, dynamic> toJson() => _$IDCardPDF417DocumentToJson(this);
}

/// ID card PDF417 document field
@JsonSerializable(
    createFactory: true, includeIfNull: false, explicitToJson: true)
class IDCardPDF417DocumentField {
  /// Field type
  IDCardPDF417DocumentFieldType type;

  /// Field value
  String value;

  IDCardPDF417DocumentField(this.type, this.value);

  factory IDCardPDF417DocumentField.fromJson(Map<String, dynamic> json) =>
      _$IDCardPDF417DocumentFieldFromJson(json);

  Map<String, dynamic> toJson() => _$IDCardPDF417DocumentFieldToJson(this);
}

/// ID card field types
enum IDCardPDF417DocumentFieldType {
  FirstName,
  LastName,
  DocumentCode,
  BirthDate,
  DateIssued,
  DateExpired,
  Optional1
}
