import 'package:json_annotation/json_annotation.dart';
import 'package:scanbot_sdk/barcode_parsers/barcode_scanner.dart';

part 'medical_certificate.g.dart';

/// Disability certificate parsed barcode document
@JsonSerializable(
    createFactory: true, includeIfNull: false, explicitToJson: true)
class MedicalCertificateDocument extends BarcodeFormattedResult {
  static const String DOCUMENT_FORMAT = 'MedicalCertificate';

  /// Fields in parsed document
  List<MedicalCertificateDocumentField> fields;

  MedicalCertificateDocument(this.fields) : super(DOCUMENT_FORMAT);

  factory MedicalCertificateDocument.fromJson(Map<String, dynamic> json) =>
      _$MedicalCertificateDocumentFromJson(json);

  @override
  Map<String, dynamic> toJson() => _$MedicalCertificateDocumentToJson(this);
}

/// Disability certificate document field
@JsonSerializable(
    createFactory: true, includeIfNull: false, explicitToJson: true)
class MedicalCertificateDocumentField {
  /// Field type
  MedicalCertificateDocumentFieldType type;

  /// Field value
  String value;

  MedicalCertificateDocumentField(this.type, this.value);

  factory MedicalCertificateDocumentField.fromJson(Map<String, dynamic> json) =>
      _$MedicalCertificateDocumentFieldFromJson(json);

  Map<String, dynamic> toJson() =>
      _$MedicalCertificateDocumentFieldToJson(this);
}

/// DC document field types
enum MedicalCertificateDocumentFieldType {
  RequiresCare,
  Accident,
  InitialCertificate,
  RenewedCertificate,
  WorkAccident,
  AssignedToAccidentInsuranceDoctor,
  IncapableOfWorkSince,
  IncapableOfWorkUntil,
  DiagnosedOn,
  DocumentDate,
  BirthDate,
  FirstName,
  LastName,
  Diagnose,
  HealthInsuranceNumber,
  InsuredPersonNumber,
  Status,
  PlaceOfOperationNumber,
  DoctorNumber,
  ChildNeedsCareFrom,
  ChildNeedsCareUntil
}
