import 'package:json_annotation/json_annotation.dart';
import 'package:scanbot_sdk/barcode_parsers/barcode_scanner.dart';

part 'sepa.g.dart';

/// SEPA barcode parsed document
@JsonSerializable(
    createFactory: true, includeIfNull: false, explicitToJson: true)
class SEPADocument extends BarcodeFormattedResult {
  static const String DOCUMENT_FORMAT = 'SEPA';

  /// Document fields */
  List<SEPADocumentField> fields;

  SEPADocument(this.fields) : super(DOCUMENT_FORMAT);

  factory SEPADocument.fromJson(Map<String, dynamic> json) =>
      _$SEPADocumentFromJson(json);

  @override
  Map<String, dynamic> toJson() => _$SEPADocumentToJson(this);
}

/// SEPA document field
@JsonSerializable(
    createFactory: true, includeIfNull: false, explicitToJson: true)
class SEPADocumentField {
  /// Field type
  SEPADocumentFieldType type = SEPADocumentFieldType.ServiceTag;

  /// Field value
  String value;

  factory SEPADocumentField.fromJson(Map<String, dynamic> json) =>
      _$SEPADocumentFieldFromJson(json);

  SEPADocumentField(this.type, this.value);

  Map<String, dynamic> toJson() => _$SEPADocumentFieldToJson(this);
}

/// SEPA document field types
enum SEPADocumentFieldType {
  ServiceTag,
  Version,
  CharacterSet,
  Identification,
  ReceiverIBAN,
  ReceiverBIC,
  ReceiverName,
  Amount,
  Reference,
  Hint,
  Purpose,
  Remittance,
  Information
}
