import 'dart:convert';

import 'package:flutter/services.dart';
import 'package:flutter/widgets.dart';
import 'package:logging/logging.dart';
import 'package:scanbot_sdk/classical_components/camera_configuration.dart';

import '../barcode_scanning_data.dart';
import '../scanbot_sdk_models.dart';
import 'classical_camera.dart';

abstract class LiveDetector {
  late MethodChannel detectorChannel;

  DetectorType get detectorType;

  @protected
  bool enabled = true;

  Map<String, dynamic> arguments = {};

  void registerResultHandler();

  attach(ScanbotCameraController controller) {
    try {
      detectorChannel =
          MethodChannel("scanbot_sdk_camera.detector_${controller.viewId}");
      registerResultHandler();
    } catch (e) {
      Logger.root.severe(e);
    }
  }

  pauseDetection() {
    enabled = false;
    detectorChannel.invokeMethod(
      "disable",
    );
  }

  resumeDetection() {
    enabled = true;
    detectorChannel.invokeMethod(
      "enable",
    );
  }
}
