import 'package:json_annotation/json_annotation.dart';

part 'create_tiff_data.g.dart';

enum TiffCompression {
  NONE,
  CCITTRLE,
  CCITTFAX3,
  CCITT_T4,
  CCITTFAX4,
  CCITT_T6,
  CCITTRLEW,
  LZW,
  PACKBITS,
  DEFLATE,
  ADOBE_DEFLATE,
}

class PredefinedCompressionTypes {
  /// return A list of compression types, which are compatible only with binarized images.
  static List<TiffCompression> binarizedOnlyCompressionTypes() {
    return [
      TiffCompression.NONE,
      TiffCompression.CCITTRLE,
      TiffCompression.CCITTFAX3,
      TiffCompression.CCITT_T4,
      TiffCompression.CCITTFAX4,
      TiffCompression.CCITT_T6,
      TiffCompression.CCITTRLEW,
    ];
  }
}

/// TIFF creation options
@JsonSerializable(includeIfNull: false, explicitToJson: true)
class TiffCreationOptions {
  /// Flag to create binarized (one bit encoded) black-and-white images.
  /// If true, the input images will be binarized and the output TIFF file will be saved with one bit per pixel.
  /// If false, the input images will be stored as-is. The default value is true.
  bool binarized;

  /// Dots Per Inches. Default value is 200dpi.
  int dpi;

  /// TIFF compression type. Default value is "LZW".
  /// Please note that some compression types are only compatible for binarized images (1-bit encoded black & white images)!
  TiffCompression compression;

  TiffCreationOptions(
      {this.binarized = true,
      this.dpi = 200,
      this.compression = TiffCompression.LZW});

  factory TiffCreationOptions.fromJson(Map<String, dynamic> json) =>
      _$TiffCreationOptionsFromJson(json);

  Map<String, dynamic> toJson() => _$TiffCreationOptionsToJson(this);
}
