import 'dart:convert';
import 'dart:io';
import 'dart:ui';

import 'package:json_annotation/json_annotation.dart';
import 'package:scanbot_sdk/sdk_utils.dart';
import 'common_data.dart';

part 'generic_document_recognizer.g.dart';

enum RootDocumentType {
  DeIdCardFront,
  DeIdCardBack,
  DePassport,
  DeDriverLicenseFront,
  DeDriverLicenseBack,
}

enum UIInterfaceOrientationMask {
  PORTRAIT,
  LANDSCAPE_LEFT,
  LANDSCAPE_RIGHT,
  PORTRAIT_UPSIDE_DOWN,
  LANDSCAPE,
  ALL,
  ALL_BUT_UPSIDE_DOWN,
}

enum CameraModule {
  BACK,
  FRONT,
}

enum Status {
  OK,
  CANCELED,
}

/// Generic Document Recognizer Result Returns fields value as Generic Document Field
@JsonSerializable(includeIfNull: false, explicitToJson: true)
class GenericDocumentField {
  String? text;
  double? confidence;
  GenericDocumentField({
    this.text,
    this.confidence,
  });

  factory GenericDocumentField.fromJson(Map<String, dynamic> json) =>
      _$GenericDocumentFieldFromJson(json);
  factory GenericDocumentField.empty() =>
      GenericDocumentField(text: "", confidence: 0);

  Map<String, dynamic> toJson() => _$GenericDocumentFieldToJson(this);
}

/// German Driver License Result from Generic Document Recognizer
@JsonSerializable(includeIfNull: false, explicitToJson: true)
class DeDriverLicenseResult {
  GenericDocumentField? birthDate;
  GenericDocumentField? birthplace;
  GenericDocumentField? expiryDate;
  GenericDocumentField? givenNames;
  GenericDocumentField? id;
  GenericDocumentField? issueDate;
  GenericDocumentField? issuingAuthority;
  GenericDocumentField? licenseCategories;
  GenericDocumentField? surname;
  GenericDocumentField? restrictions;
  DriverLicenseCategories? categories;

  DeDriverLicenseResult({
    this.birthDate,
    this.birthplace,
    this.expiryDate,
    this.givenNames,
    this.id,
    this.issueDate,
    this.issuingAuthority,
    this.licenseCategories,
    this.surname,
    this.restrictions,
    this.categories,
  });
  factory DeDriverLicenseResult.fromJson(Map<String, dynamic> json) =>
      Platform.isIOS
          ? DeDriverLicenseResultFromJson(json)
          : _$DeDriverLicenseResultFromJson(json);

  Map<String, dynamic> toJson() => _$DeDriverLicenseResultToJson(this);
}

DeDriverLicenseResult DeDriverLicenseResultFromJson(Map<String, dynamic> json) {
  return DeDriverLicenseResult(
    birthDate: json['birthdate'] == null
        ? GenericDocumentField.empty()
        : GenericDocumentField.fromJson(
            json['birthdate'] as Map<String, dynamic>),
    birthplace: json['birthplace'] == null
        ? GenericDocumentField.empty()
        : GenericDocumentField.fromJson(
            json['birthplace'] as Map<String, dynamic>),
    expiryDate: json['expirydate'] == null
        ? GenericDocumentField.empty()
        : GenericDocumentField.fromJson(
            json['expirydate'] as Map<String, dynamic>),
    givenNames: json['givennames'] == null
        ? GenericDocumentField.empty()
        : GenericDocumentField.fromJson(
            json['givennames'] as Map<String, dynamic>),
    id: json['id'] == null
        ? GenericDocumentField.empty()
        : GenericDocumentField.fromJson(json['id'] as Map<String, dynamic>),
    issueDate: json['issuedate'] == null
        ? GenericDocumentField.empty()
        : GenericDocumentField.fromJson(
            json['issuedate'] as Map<String, dynamic>),
    issuingAuthority: json['issuingauthority'] == null
        ? GenericDocumentField.empty()
        : GenericDocumentField.fromJson(
            json['issuingauthority'] as Map<String, dynamic>),
    licenseCategories: json['licensecategories'] == null
        ? GenericDocumentField.empty()
        : GenericDocumentField.fromJson(
            json['licensecategories'] as Map<String, dynamic>),
    surname: json['surname'] == null
        ? GenericDocumentField.empty()
        : GenericDocumentField.fromJson(
            json['surname'] as Map<String, dynamic>),
    restrictions: json['restrictions'] == null
        ? GenericDocumentField.empty()
        : GenericDocumentField.fromJson(
            json['restrictions'] as Map<String, dynamic>),
    categories: json['categories'] == null
        ? DriverLicenseCategories.empty()
        : DriverLicenseCategories.fromJson(
            json['categories'] as Map<String, dynamic>),
  );
}

/// Driver License Categories of German Driver License from Result of Generic Document Recognizer
@JsonSerializable(includeIfNull: false, explicitToJson: true)
class DriverLicenseCategories {
  DriverLicenseCategory? a;
  DriverLicenseCategory? a1;
  DriverLicenseCategory? a2;
  DriverLicenseCategory? b;
  DriverLicenseCategory? b1;
  DriverLicenseCategory? be;
  DriverLicenseCategory? c;
  DriverLicenseCategory? c1;
  DriverLicenseCategory? c1e;
  DriverLicenseCategory? ce;
  DriverLicenseCategory? d;
  DriverLicenseCategory? d1;
  DriverLicenseCategory? d1e;
  DriverLicenseCategory? de;
  DriverLicenseCategory? l;
  DriverLicenseCategory? m;
  DriverLicenseCategory? t;
  DriverLicenseCategories({
    this.a,
    this.a1,
    this.a2,
    this.b,
    this.b1,
    this.be,
    this.c,
    this.c1,
    this.c1e,
    this.ce,
    this.d,
    this.d1,
    this.d1e,
    this.de,
    this.l,
    this.m,
    this.t,
  });
  factory DriverLicenseCategories.fromJson(Map<String, dynamic> json) =>
      _$DriverLicenseCategoriesFromJson(json);

  factory DriverLicenseCategories.empty() => DriverLicenseCategories(
        a: DriverLicenseCategory.empty(),
        a1: DriverLicenseCategory.empty(),
        a2: DriverLicenseCategory.empty(),
        b: DriverLicenseCategory.empty(),
        b1: DriverLicenseCategory.empty(),
        be: DriverLicenseCategory.empty(),
        c: DriverLicenseCategory.empty(),
        c1: DriverLicenseCategory.empty(),
        c1e: DriverLicenseCategory.empty(),
        ce: DriverLicenseCategory.empty(),
        d: DriverLicenseCategory.empty(),
        de: DriverLicenseCategory.empty(),
        d1: DriverLicenseCategory.empty(),
        d1e: DriverLicenseCategory.empty(),
        l: DriverLicenseCategory.empty(),
        m: DriverLicenseCategory.empty(),
        t: DriverLicenseCategory.empty(),
      );

  Map<String, dynamic> toJson() => _$DriverLicenseCategoriesToJson(this);
}

/// Driver License Category of German Driver License from Result of Generic Document Recognizer
@JsonSerializable(includeIfNull: false, explicitToJson: true)
class DriverLicenseCategory {
  GenericDocumentField? restrictions;
  GenericDocumentField? validFrom;
  GenericDocumentField? validUntil;

  DriverLicenseCategory({
    this.restrictions,
    this.validFrom,
    this.validUntil,
  });

  factory DriverLicenseCategory.fromJson(Map<String, dynamic> json) =>
      _$DriverLicenseCategoryFromJson(json);

  factory DriverLicenseCategory.empty() => DriverLicenseCategory(
        restrictions: GenericDocumentField.empty(),
        validFrom: GenericDocumentField.empty(),
        validUntil: GenericDocumentField.empty(),
      );

  Map<String, dynamic> toJson() => _$DriverLicenseCategoryToJson(this);
}

/// German Passport Result from Generic Document Recognizer
@JsonSerializable(includeIfNull: false, explicitToJson: true)
class DePassportResult {
  GenericDocumentField? birthDate;
  GenericDocumentField? birthplace;
  GenericDocumentField? countryCode;
  GenericDocumentField? expiryDate;
  GenericDocumentField? gender;
  GenericDocumentField? givenNames;
  GenericDocumentField? id;
  GenericDocumentField? issueDate;
  GenericDocumentField? issuingAuthority;
  GenericDocumentField? maidenName;
  GenericDocumentField? nationality;
  GenericDocumentField? passportType;
  GenericDocumentField? rawMrz;
  GenericDocumentField? surname;
  MrzDocumentResult? mrz;

  DePassportResult({
    this.birthDate,
    this.birthplace,
    this.countryCode,
    this.expiryDate,
    this.gender,
    this.givenNames,
    this.id,
    this.issueDate,
    this.issuingAuthority,
    this.maidenName,
    this.nationality,
    this.passportType,
    this.rawMrz,
    this.surname,
    this.mrz,
  });

  factory DePassportResult.fromJson(Map<String, dynamic> json) => Platform.isIOS
      ? DePassportResultFromJson(json)
      : _$DePassportResultFromJson(json);

  Map<String, dynamic> toJson() => _$DePassportResultToJson(this);
}

DePassportResult DePassportResultFromJson(Map<String, dynamic> json) {
  return DePassportResult(
    birthDate: json['birthdate'] == null
        ? GenericDocumentField.empty()
        : GenericDocumentField.fromJson(
            json['birthdate'] as Map<String, dynamic>),
    birthplace: json['birthplace'] == null
        ? GenericDocumentField.empty()
        : GenericDocumentField.fromJson(
            json['birthplace'] as Map<String, dynamic>),
    countryCode: json['countrycode'] == null
        ? GenericDocumentField.empty()
        : GenericDocumentField.fromJson(
            json['countrycode'] as Map<String, dynamic>),
    expiryDate: json['expirydate'] == null
        ? GenericDocumentField.empty()
        : GenericDocumentField.fromJson(
            json['expirydate'] as Map<String, dynamic>),
    gender: json['gender'] == null
        ? GenericDocumentField.empty()
        : GenericDocumentField.fromJson(json['gender'] as Map<String, dynamic>),
    givenNames: json['givennames'] == null
        ? GenericDocumentField.empty()
        : GenericDocumentField.fromJson(
            json['givennames'] as Map<String, dynamic>),
    id: json['id'] == null
        ? GenericDocumentField.empty()
        : GenericDocumentField.fromJson(json['id'] as Map<String, dynamic>),
    issueDate: json['issuedate'] == null
        ? GenericDocumentField.empty()
        : GenericDocumentField.fromJson(
            json['issuedate'] as Map<String, dynamic>),
    issuingAuthority: json['issuingauthority'] == null
        ? GenericDocumentField.empty()
        : GenericDocumentField.fromJson(
            json['issuingauthority'] as Map<String, dynamic>),
    maidenName: json['maidenname'] == null
        ? GenericDocumentField.empty()
        : GenericDocumentField.fromJson(
            json['maidenname'] as Map<String, dynamic>),
    nationality: json['nationality'] == null
        ? GenericDocumentField.empty()
        : GenericDocumentField.fromJson(
            json['nationality'] as Map<String, dynamic>),
    passportType: json['passporttype'] == null
        ? GenericDocumentField.empty()
        : GenericDocumentField.fromJson(
            json['passporttype'] as Map<String, dynamic>),
    rawMrz: json['rawmrz'] == null
        ? GenericDocumentField.empty()
        : GenericDocumentField.fromJson(json['rawmrz'] as Map<String, dynamic>),
    surname: json['surname'] == null
        ? GenericDocumentField.empty()
        : GenericDocumentField.fromJson(
            json['surname'] as Map<String, dynamic>),
    mrz: json['mrz'] == null
        ? null
        : MrzDocumentResult.fromJson(json['mrz'] as Map<String, dynamic>),
  );
}

/// Mrz of German Passport of Generic Document Recognizer Result
@JsonSerializable(includeIfNull: false, explicitToJson: true)
class MrzDocumentResult {
  GenericDocumentField? birthDate;
  List<GenericDocumentField>? checkDigits;
  GenericDocumentField? documentNumber;
  GenericDocumentField? expiryDate;
  GenericDocumentField? gender;
  GenericDocumentField? givenNames;
  GenericDocumentField? issuingAuthority;
  GenericDocumentField? nationality;
  GenericDocumentField? optional1;
  GenericDocumentField? optional2;
  GenericDocumentField? surname;
  GenericDocumentField? travelDocType;
  GenericDocumentField? travelDocTypeVariant;
  MrzDocumentResult({
    this.birthDate,
    this.checkDigits,
    this.documentNumber,
    this.expiryDate,
    this.gender,
    this.givenNames,
    this.issuingAuthority,
    this.nationality,
    this.optional1,
    this.optional2,
    this.surname,
    this.travelDocType,
    this.travelDocTypeVariant,
  });

  factory MrzDocumentResult.fromJson(Map<String, dynamic> json) =>
      Platform.isIOS
          ? MrzDocumentResultFromJson(json)
          : _$MrzDocumentResultFromJson(json);

  factory MrzDocumentResult.empty() => MrzDocumentResult(
        birthDate: GenericDocumentField.empty(),
        checkDigits: [],
        documentNumber: GenericDocumentField.empty(),
        expiryDate: GenericDocumentField.empty(),
        gender: GenericDocumentField.empty(),
        givenNames: GenericDocumentField.empty(),
        issuingAuthority: GenericDocumentField.empty(),
        nationality: GenericDocumentField.empty(),
        optional1: GenericDocumentField.empty(),
        optional2: GenericDocumentField.empty(),
        surname: GenericDocumentField.empty(),
        travelDocType: GenericDocumentField.empty(),
        travelDocTypeVariant: GenericDocumentField.empty(),
      );

  Map<String, dynamic> toJson() => _$MrzDocumentResultToJson(this);
}

MrzDocumentResult MrzDocumentResultFromJson(Map<String, dynamic> json) {
  return MrzDocumentResult(
    birthDate: json['birthdate'] == null
        ? GenericDocumentField.empty()
        : GenericDocumentField.fromJson(
            json['birthdate'] as Map<String, dynamic>),
    checkDigits: (json['checkdigits'] as List<dynamic>?)
        ?.map((e) => GenericDocumentField.fromJson(e as Map<String, dynamic>))
        .toList(),
    documentNumber: json['documentnumber'] == null
        ? GenericDocumentField.empty()
        : GenericDocumentField.fromJson(
            json['documentnumber'] as Map<String, dynamic>),
    expiryDate: json['expirydate'] == null
        ? GenericDocumentField.empty()
        : GenericDocumentField.fromJson(
            json['expirydate'] as Map<String, dynamic>),
    gender: json['gender'] == null
        ? GenericDocumentField.empty()
        : GenericDocumentField.fromJson(json['gender'] as Map<String, dynamic>),
    givenNames: json['givennames'] == null
        ? GenericDocumentField.empty()
        : GenericDocumentField.fromJson(
            json['givennames'] as Map<String, dynamic>),
    issuingAuthority: json['issuingauthority'] == null
        ? GenericDocumentField.empty()
        : GenericDocumentField.fromJson(
            json['issuingauthority'] as Map<String, dynamic>),
    nationality: json['nationality'] == null
        ? GenericDocumentField.empty()
        : GenericDocumentField.fromJson(
            json['nationality'] as Map<String, dynamic>),
    optional1: json['optional1'] == null
        ? GenericDocumentField.empty()
        : GenericDocumentField.fromJson(
            json['optional1'] as Map<String, dynamic>),
    optional2: json['optional2'] == null
        ? GenericDocumentField.empty()
        : GenericDocumentField.fromJson(
            json['optional2'] as Map<String, dynamic>),
    surname: json['surname'] == null
        ? GenericDocumentField.empty()
        : GenericDocumentField.fromJson(
            json['surname'] as Map<String, dynamic>),
    travelDocType: json['traveldoctype'] == null
        ? GenericDocumentField.empty()
        : GenericDocumentField.fromJson(
            json['traveldoctype'] as Map<String, dynamic>),
    travelDocTypeVariant: json['traveldoctypevariant'] == null
        ? GenericDocumentField.empty()
        : GenericDocumentField.fromJson(
            json['traveldoctypevariant'] as Map<String, dynamic>),
  );
}

/// German Id Card Result from Generic Document Recognizer
@JsonSerializable(includeIfNull: false, explicitToJson: true)
class DeIdCardResult {
  GenericDocumentField? birthDate;
  GenericDocumentField? birthplace;
  GenericDocumentField? expiryDate;
  GenericDocumentField? givenNames;
  GenericDocumentField? id;
  GenericDocumentField? maidenName;
  GenericDocumentField? nationality;
  GenericDocumentField? pin;
  GenericDocumentField? surname;
  GenericDocumentField? address;
  GenericDocumentField? eyeColor;
  GenericDocumentField? height;
  GenericDocumentField? issueDate;
  GenericDocumentField? issuingAuthority;
  GenericDocumentField? pseudonym;
  GenericDocumentField? rawMrz;
  DeIdCardResult({
    this.birthDate,
    this.birthplace,
    this.expiryDate,
    this.givenNames,
    this.id,
    this.maidenName,
    this.nationality,
    this.pin,
    this.surname,
    this.address,
    this.eyeColor,
    this.height,
    this.issueDate,
    this.issuingAuthority,
    this.pseudonym,
    this.rawMrz,
  });
  factory DeIdCardResult.fromJson(Map<String, dynamic> json) => Platform.isIOS
      ? DeIdCardResultFromJson(json)
      : _$DeIdCardResultFromJson(json);

  Map<String, dynamic> toJson() => _$DeIdCardResultToJson(this);
}

DeIdCardResult DeIdCardResultFromJson(Map<String, dynamic> json) {
  return DeIdCardResult(
    birthDate: json['birthdate'] == null
        ? GenericDocumentField.empty()
        : GenericDocumentField.fromJson(
            json['birthdate'] as Map<String, dynamic>),
    birthplace: json['birthplace'] == null
        ? GenericDocumentField.empty()
        : GenericDocumentField.fromJson(
            json['birthplace'] as Map<String, dynamic>),
    expiryDate: json['expirydate'] == null
        ? GenericDocumentField.empty()
        : GenericDocumentField.fromJson(
            json['expirydate'] as Map<String, dynamic>),
    givenNames: json['givennames'] == null
        ? GenericDocumentField.empty()
        : GenericDocumentField.fromJson(
            json['givennames'] as Map<String, dynamic>),
    id: json['id'] == null
        ? GenericDocumentField.empty()
        : GenericDocumentField.fromJson(json['id'] as Map<String, dynamic>),
    maidenName: json['maidenname'] == null
        ? GenericDocumentField.empty()
        : GenericDocumentField.fromJson(
            json['maidenname'] as Map<String, dynamic>),
    nationality: json['nationality'] == null
        ? GenericDocumentField.empty()
        : GenericDocumentField.fromJson(
            json['nationality'] as Map<String, dynamic>),
    pin: json['pin'] == null
        ? GenericDocumentField.empty()
        : GenericDocumentField.fromJson(json['pin'] as Map<String, dynamic>),
    surname: json['surname'] == null
        ? GenericDocumentField.empty()
        : GenericDocumentField.fromJson(
            json['surname'] as Map<String, dynamic>),
    address: json['address'] == null
        ? GenericDocumentField.empty()
        : GenericDocumentField.fromJson(
            json['address'] as Map<String, dynamic>),
    eyeColor: json['eyecolor'] == null
        ? GenericDocumentField.empty()
        : GenericDocumentField.fromJson(
            json['eyecolor'] as Map<String, dynamic>),
    height: json['height'] == null
        ? GenericDocumentField.empty()
        : GenericDocumentField.fromJson(json['height'] as Map<String, dynamic>),
    issueDate: json['issuedate'] == null
        ? GenericDocumentField.empty()
        : GenericDocumentField.fromJson(
            json['issuedate'] as Map<String, dynamic>),
    issuingAuthority: json['issuingauthority'] == null
        ? GenericDocumentField.empty()
        : GenericDocumentField.fromJson(
            json['issuingauthority'] as Map<String, dynamic>),
    pseudonym: json['pseudonym'] == null
        ? GenericDocumentField.empty()
        : GenericDocumentField.fromJson(
            json['pseudonym'] as Map<String, dynamic>),
    rawMrz: json['rawmrz'] == null
        ? GenericDocumentField.empty()
        : GenericDocumentField.fromJson(json['rawmrz'] as Map<String, dynamic>),
  );
}

enum GenericDocumentType {
  DeDriverLicense,
  DePassport,
  DeIdCard,
}

/// Generic Document Recognizer Result
@JsonSerializable(includeIfNull: false, explicitToJson: true)
class GenericDocumentRecognizerResult {
  GenericDocumentType? documentType;
  Status? status;
  dynamic fields;
  GenericDocumentRecognizerResult({
    this.documentType,
    this.status,
    this.fields,
  });

  factory GenericDocumentRecognizerResult.fromJson(Map<String, dynamic> json) =>
      GenericDocumentRecognizerResultFromJson(json);

  Map<String, dynamic> toJson() =>
      _$GenericDocumentRecognizerResultToJson(this);
}

GenericDocumentRecognizerResult GenericDocumentRecognizerResultFromJson(
    Map<String, dynamic> json) {
  GenericDocumentType? documentType;
  dynamic fields;
  Status status;
  if (json['status'] == 'OK') {
    status = Status.OK;
    switch (json['documentType']) {
      case "DeDriverLicense":
      case "DeDriverLicenseFront":
      case "DeDriverLicenseBack":
        documentType = GenericDocumentType.DeDriverLicense;
        break;
      case "DePassport":
        documentType = GenericDocumentType.DePassport;
        break;
      case "DeIdCard":
      case "DeIdCardFront":
      case "DeIdCardBack":
        documentType = GenericDocumentType.DeIdCard;
        break;
    }
    if (documentType == GenericDocumentType.DeDriverLicense) {
      fields = DeDriverLicenseResult.fromJson(
          Platform.isAndroid ? json['fields'] : jsonDecode(json['fields']));
    } else if (documentType == GenericDocumentType.DePassport) {
      fields = DePassportResult.fromJson(
          Platform.isAndroid ? json['fields'] : jsonDecode(json['fields']));
    } else if (documentType == GenericDocumentType.DeIdCard) {
      fields = DeIdCardResult.fromJson(
          Platform.isAndroid ? json['fields'] : jsonDecode(json['fields']));
    } else {
      throw ArgumentError.value(fields);
    }
  } else {
    status = Status.CANCELED;
  }

  return GenericDocumentRecognizerResult(
    documentType: documentType,
    status: status,
    fields: fields,
  );
}

/// Generic Document Recognizer class. All UI configuration options are optional.
@JsonSerializable(includeIfNull: false, explicitToJson: true)
class GenericDocumentRecognizerConfiguration {
  /// Accepted document types. All other document types will be ignored.
  /// All by default - DeIdFront, DeIdBack, DePassport, DeDriverLicenseFront, and DeDriverLicenseBack
  List<RootDocumentType>? acceptedDocumentTypes;

  /// Background color of the detection overlay.
  @JsonKey(fromJson: colorFromString, toJson: toColorString)
  Color? cameraOverlayColor;

  /// Whether the cancel button is hidden or not. (iOS only)
  bool? cancelButtonHidden;

  /// String being displayed on the cancel button.
  String? cancelButtonTitle;

  /// String being displayed on the clear button.
  String? clearButtonTitle;

  /// String that shows average confidence value of scanned card.
  String? confidenceValue;

  /// Foreground color of the top bar buttons on the details screen.
  @JsonKey(fromJson: colorFromString, toJson: toColorString)
  Color? detailsActionColor;

  /// Background color of the details screen.
  @JsonKey(fromJson: colorFromString, toJson: toColorString)
  Color? detailsBackgroundColor;

  /// Text color in the details list. Also affects image background and separator.
  @JsonKey(fromJson: colorFromString, toJson: toColorString)
  Color? detailsPrimaryColor;

  /// Foreground color of the detection overlay.
  @JsonKey(fromJson: colorFromString, toJson: toColorString)
  Color? finderLineColor;

  /// Width of finder frame border. Default is 2.
  int? finderLineWidth;

  /// Whether the torch light is toggled on or off.
  bool? flashEnabled;

  /// A title to show image content. Android Only
  String? imageTitle;

  /// String that notifies that nothing was scanned yet.
  String? noDataTitle;

  /// String that asks user to scan back side of a card.
  String? scanBackSideTitle;

  /// String that asks user to scan front side of a card.
  String? scanFrontSideTitle;

  /// String that notifies that both sides of card are scanned.
  String? scannedEverythingTitle;

  /// String that asks user to start scanning a card.
  String? startScanningTitle;

  /// String being displayed on the submit button.
  String? viewResultsButtonTitle;

  /// Color of tip background on scanning screen.
  @JsonKey(fromJson: colorFromString, toJson: toColorString)
  Color? tipBackgroundColor;

  /// Color of tip text on scanning screen.
  @JsonKey(fromJson: colorFromString, toJson: toColorString)
  Color? tipTextColor;

  /// Background color of the top bar.
  @JsonKey(fromJson: colorFromString, toJson: toColorString)
  Color? topBarBackgroundColor;

  /// Foreground color of the top bar buttons on the scanning screen.
  @JsonKey(fromJson: colorFromString, toJson: toColorString)
  Color? topBarButtonsColor;

  /// Foreground color of the flash button when flash is off.
  @JsonKey(fromJson: colorFromString, toJson: toColorString)
  Color? topBarButtonsInactiveColor;

  /// Controls whether buttons should use all capitals style, as defined by the Android Material Design. Defaults to TRUE.
  /// Android only.
  bool? useButtonsAllCaps;

  /// The accepted minimal sharpness score. Images with a score less than that will
  /// be rejected with blurry status.
  ///
  /// 0 - any image will be accepted.
  /// 80 - a good compromise; the recommended setting.
  /// 100 - only very sharp images will be accepted.
  ///
  /// The default value is 80.
  int? sharpnessAcceptanceFactor;

  /// Orientation lock mode of the UI, the camera preview, and the output images.
  /// By default the orientation is not locked.
  CameraOrientationMode? orientationLockMode;

  /// Set supported camera module: "BACK", "FRONT".
  /// Optional. By default is "BACK".
  CameraModule? cameraModule;

  /// Text that will be displayed when the app
  /// is not allowed to use the camera, prompting the user
  /// to enable the usage of the camera.
  String? enableCameraExplanationText;

  /// Title of the button that opens the screen where the user can allow
  /// the usage of the camera by the app.
  String? enableCameraButtonTitle;

  ///Configuration of document fields shown in details screen
  GenericDocumentFieldConfiguration? detailsFieldConfiguration;

  GenericDocumentRecognizerConfiguration({
    this.acceptedDocumentTypes,
    this.cameraOverlayColor,
    this.cancelButtonHidden,
    this.cancelButtonTitle,
    this.clearButtonTitle,
    this.confidenceValue,
    this.detailsActionColor,
    this.detailsBackgroundColor,
    this.detailsPrimaryColor,
    this.finderLineColor,
    this.finderLineWidth,
    this.flashEnabled,
    this.imageTitle,
    this.noDataTitle,
    this.scanBackSideTitle,
    this.scanFrontSideTitle,
    this.scannedEverythingTitle,
    this.startScanningTitle,
    this.viewResultsButtonTitle,
    this.tipBackgroundColor,
    this.tipTextColor,
    this.topBarBackgroundColor,
    this.topBarButtonsColor,
    this.topBarButtonsInactiveColor,
    this.useButtonsAllCaps,
    this.sharpnessAcceptanceFactor,
    this.orientationLockMode,
    this.cameraModule,
    this.enableCameraExplanationText,
    this.enableCameraButtonTitle,
    this.detailsFieldConfiguration,
  });

  factory GenericDocumentRecognizerConfiguration.fromJson(
          Map<String, dynamic> json) =>
      _$GenericDocumentRecognizerConfigurationFromJson(json);

  Map<String, dynamic> toJson() =>
      _$GenericDocumentRecognizerConfigurationToJson(this);
}

/// Generic Document Field Configration class. All UI configuration options are optional.
@JsonSerializable(includeIfNull: false, explicitToJson: true)
class GenericDocumentFieldConfiguration {
  ///A title for IdCardBack in details screen.
  String? deIdCardBackDocumentTitle;

  ///A title for IdCardFront in details screen.
  String? deIdCardFrontDocumentTitle;

  ///A title for Passport in details screen.
  String? dePassportDocumentTitle;

  ///A title for DriverLicenseBack in details screen.
  String? deDriverLicenseBackDocumentTitle;

  ///A title for DriverLicenseFront in details screen.
  String? deDriverLicenseFrontDocumentTitle;

  ///A title for address field in details screen.
  String? fieldAddressTitle;

  ///A title for birth date field in details screen.
  String? fieldBirthDateTitle;

  ///A title for birth place field in details screen.
  String? fieldBirthPlaceTitle;

  ///Color of confidence value label background in details screen, when the field confidence level is high.
  @JsonKey(fromJson: colorFromString, toJson: toColorString)
  Color? fieldConfidenceHighColor;

  ///Color of confidence value label background  in details screen, when the field confidence level is low.
  @JsonKey(fromJson: colorFromString, toJson: toColorString)
  Color? fieldConfidenceLowColor;

  ///Color of confidence value label background  in details screen, when the field confidence level is moderate.
  @JsonKey(fromJson: colorFromString, toJson: toColorString)
  Color? fieldConfidenceModerateColor;

  ///Color of confidence value label text in details.
  @JsonKey(fromJson: colorFromString, toJson: toColorString)
  Color? fieldConfidenceTextColor;

  ///A title for country code field in details screen.
  String? fieldCountryCodeTitle;

  ///A title for expiry date field in details screen.
  String? fieldExpiryDateTitle;

  ///A title for eye color field in details screen.
  String? fieldEyeColorTitle;

  ///A title for gender field in details screen.
  String? fieldGenderTitle;

  ///A title for given names field in details screen.
  String? fieldGivenNamesTitle;

  ///A title for height field in details screen.
  String? fieldHeightTitle;

  ///A title for id field in details screen.
  String? fieldIdTitle;

  ///A title for issuing date field in details screen.
  String? fieldIssueDateTitle;

  ///A title for issuing authority  field in details screen.
  String? fieldIssuingAuthorityTitle;

  ///A title for maiden name field in details screen.
  String? fieldMaidenNameTitle;

  ///A title for machine readable zone field in details screen.
  String? fieldRawMrzTitle;

  ///A title for nationality field in details screen.
  String? fieldNationalityTitle;

  ///A title for passport type field in details screen.
  String? fieldPassportTypeTitle;

  ///A title for photo field in details screen.
  String? fieldPhotoTitle;

  ///A title for PIN field in details screen.
  String? fieldPinTitle;

  ///A title for pseudonym field in details screen.
  String? fieldPseudonymTitle;

  ///String used for displaying amount of detected fields. Use %d for number formatting symbol.
  String? fieldsCountText;

  ///Text color of the fields count label.
  String? fieldsCountTextColor;

  ///A title for signature field in details screen.
  String? fieldSignatureTitle;

  ///A title for surname field in details screen.
  String? fieldSurnameTitle;

  ///A title for restrictions for the driver's license in details screen.
  String? fieldRestrictionsTitle;

  ///A title for driver's license categories in details screen.
  String? fieldLicenseCategoriesTitle;

  GenericDocumentFieldConfiguration({
    this.deIdCardBackDocumentTitle,
    this.deIdCardFrontDocumentTitle,
    this.dePassportDocumentTitle,
    this.deDriverLicenseBackDocumentTitle,
    this.deDriverLicenseFrontDocumentTitle,
    this.fieldAddressTitle,
    this.fieldBirthDateTitle,
    this.fieldBirthPlaceTitle,
    this.fieldConfidenceHighColor,
    this.fieldConfidenceLowColor,
    this.fieldConfidenceModerateColor,
    this.fieldConfidenceTextColor,
    this.fieldCountryCodeTitle,
    this.fieldExpiryDateTitle,
    this.fieldEyeColorTitle,
    this.fieldGenderTitle,
    this.fieldGivenNamesTitle,
    this.fieldHeightTitle,
    this.fieldIdTitle,
    this.fieldIssueDateTitle,
    this.fieldIssuingAuthorityTitle,
    this.fieldMaidenNameTitle,
    this.fieldRawMrzTitle,
    this.fieldNationalityTitle,
    this.fieldPassportTypeTitle,
    this.fieldPhotoTitle,
    this.fieldPinTitle,
    this.fieldPseudonymTitle,
    this.fieldsCountText,
    this.fieldsCountTextColor,
    this.fieldSignatureTitle,
    this.fieldSurnameTitle,
    this.fieldRestrictionsTitle,
    this.fieldLicenseCategoriesTitle,
  });

  factory GenericDocumentFieldConfiguration.fromJson(
          Map<String, dynamic> json) =>
      _$GenericDocumentFieldConfigurationFromJson(json);

  Map<String, dynamic> toJson() =>
      _$GenericDocumentFieldConfigurationToJson(this);
}
