#!/bin/bash

#
# Since flutter pub does not support a solid ignoring/excluding mechanism for files which should NOT go into the
# final pub package, we have this custom script for that. It just copies files and folders from a white list
# into a custom package folder and performs the publishing from this folder.
#

# Exit immediately if a command exits with a non-zero status.
set -e

CUSTOM_PACKAGE_FOLDER=".custom-package-preparation"

# our test pub repo server:
TEST_PUB_REPO_SERVER="http://34.253.244.29:4000"


# cleanup/re-create the custom package folder:
rm -rf $CUSTOM_PACKAGE_FOLDER
mkdir $CUSTOM_PACKAGE_FOLDER


# copy white list files and folders:
echo
echo "Preparing files and folders ..."
cp -r android $CUSTOM_PACKAGE_FOLDER/
cp -r ios $CUSTOM_PACKAGE_FOLDER/
cp -r lib $CUSTOM_PACKAGE_FOLDER/
cp -r example $CUSTOM_PACKAGE_FOLDER/
cp -r assets $CUSTOM_PACKAGE_FOLDER/
cp pubspec.yaml $CUSTOM_PACKAGE_FOLDER/
cp analysis_options.yaml $CUSTOM_PACKAGE_FOLDER/
cp README.md $CUSTOM_PACKAGE_FOLDER/
cp CHANGELOG.md $CUSTOM_PACKAGE_FOLDER/
cp LICENSE $CUSTOM_PACKAGE_FOLDER/
cp Libraries.txt $CUSTOM_PACKAGE_FOLDER/
echo "... done."
echo

# perform publishing:
cd $CUSTOM_PACKAGE_FOLDER/

read -p "Publish on PRODUCTION(!) public pub repo? [y/n]: " -n 1 -r
if [[ $REPLY =~ ^[Yy]$ ]]; then
  echo
  echo "Publishing ..."
  flutter pub pub publish
  echo "... done."
fi
echo


# and finally cleanup the custom package folder:
cd ..
rm -rf $CUSTOM_PACKAGE_FOLDER

echo
echo "DONE! 🎉"
echo
